import { useEffect, useState, useMemo } from "react";
import { OperationDetails, AwxJobDetails } from "../../store/deployApi";
import { Typography, Stack, Box, LinearProgress } from "@mui/material";
import {
  KeyValueTable,
  SimpleAccordion,
  AlertBannerList,
  ExternalLink,
  InternalLink,
  formatDateAndTime,
  EmptyValue,
  Duration,
  Step,
  AccessControl
} from "@ess-ics/ce-ui-common";
import { JobBadge } from "./JobBadge";
import { DeploymentJobOutput } from "../deployments/DeploymentJobOutput";
import { JobGitRefIcon } from "./JobTable/JobGitRefIcon";
import { JobGitRefLink } from "./JobTable/JobGitRefLink";
import { AWXJobDetails, Status } from "../../api/DataTypes";

interface JobDetailsProps {
  operation: OperationDetails;
  job: AwxJobDetails;
}

const createAlert = (type: Status, message: string) => {
  return {
    type,
    message
  };
};

const calculateHostText = (operation: OperationDetails) => {
  // host is resolvable => show link for users
  if (operation?.host?.hostId && operation?.host?.externalIdValid) {
    return (
      <InternalLink
        to={`/hosts/${operation.host.hostId}`}
        label={`Host Details, ${operation.host.fqdn}`}
      >
        {operation.host.fqdn}
      </InternalLink>
    );
  }
  // gray out hostname if it is from cache
  return <Typography color="gray">{operation?.host?.fqdn}</Typography>;
};

const getDeploymentDetails = (operation: OperationDetails) => {
  return {
    Revision: (
      <Box sx={{ display: "flex", gap: "6px", alignItems: "center" }}>
        <JobGitRefIcon
          job={operation}
          fontSize="medium"
        />
        <JobGitRefLink job={operation} />
      </Box>
    ),
    host: calculateHostText(operation),
    user: (
      <InternalLink
        to={`/user/${operation.createdBy}`}
        label={`User profile, ${operation.createdBy}`}
      >
        {operation.createdBy}
      </InternalLink>
    ),
    "AWX job link": operation?.jobUrl ? (
      <ExternalLink
        href={operation.jobUrl}
        label="AWX job"
      >
        {operation.jobUrl}
      </ExternalLink>
    ) : (
      <EmptyValue />
    ),
    "created time": operation?.createdAt ? (
      formatDateAndTime(operation.createdAt)
    ) : (
      <EmptyValue />
    ),
    "AWX job start time": operation?.startTime ? (
      formatDateAndTime(operation.startTime)
    ) : (
      <EmptyValue />
    ),
    duration: operation?.finishedAt ? (
      <Duration
        createOrStartDate={operation?.startTime ?? operation?.createdAt}
        finishedAt={operation?.finishedAt}
        textOnly
      />
    ) : (
      <EmptyValue />
    )
  };
};

export const JobDetails = ({ operation, job }: JobDetailsProps) => {
  const [alerts, setAlerts] = useState<
    { type?: Status; message?: string; link?: string }[]
  >(operation.alerts ?? []);

  const awxJob = useMemo(
    () => new AWXJobDetails(operation, operation.type),
    [operation]
  );

  useEffect(() => {
    const operationAlerts = operation.alerts ? operation.alerts : [];
    setAlerts([
      createAlert(awxJob.severity(), awxJob.message()),
      ...operationAlerts
    ]);
  }, [operation, awxJob]);

  return (
    <Stack spacing={2}>
      <Stack>{<AlertBannerList alerts={alerts} />}</Stack>
      <JobBadge operation={operation} />
      <KeyValueTable
        obj={getDeploymentDetails(operation)}
        variant="overline"
        sx={{ border: 0 }}
        valueOptions={{ headerName: "" }}
      />
      <AccessControl
        allowedRoles={["DeploymentToolAdmin", "DeploymentToolIntegrator"]}
        renderNoAccess={() => <></>}
      >
        <SimpleAccordion
          summary={
            <Stack
              flexDirection="row"
              alignItems="center"
            >
              <Typography
                component="h2"
                variant="h3"
                sx={{ marginRight: "8px" }}
              >
                Job log stream
              </Typography>
              <Step
                showToolTip
                step={{
                  state: operation?.status?.toLowerCase(),
                  label: awxJob.message()
                }}
              />
            </Stack>
          }
        >
          {job ? (
            <DeploymentJobOutput deploymentJob={job} />
          ) : (
            <LinearProgress />
          )}
        </SimpleAccordion>
      </AccessControl>
    </Stack>
  );
};
